# -*- coding: utf-8 -*-
"""
Created on Fri May 11 15:00:52 2018

@author: Xu
"""

from sklearn.cluster import KMeans
from scipy.interpolate import interp1d
import statsmodels.api as sm
import numpy as np
import pandas as pd
from scipy.sparse import diags
from scipy import arange
# median smoothing
from scipy.signal import medfilt
# plotting packages
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d
from scipy.optimize import fsolve
import os
folder_address=os.getcwd()
import shutil

# To import from self-wrote libraries
import sys
sys.path.append(folder_address+r'\Shock simulation library')
import DefineShock as ds
import chemkinsinglerun as csr

# Below are import actions that can be replaced by defining the function in this notebook, which is convenient to adjust.
#from pressure_trace_analysis import pressure_processing_2shocks
#from pressure_trace_analysis import pressure_processing_1shock
#from pressure_trace_analysis import back_cal_T
#from pressure_trace_analysis import getTavg

# Engineering Constants
#################################################################################
# Argon specific heat ratio
gamma=1.667

#Argon speed of sound at 373.15 K in m/s
a1=359.8048494

# Matsugi TFE rate constants ---- High pressure limit
Rate_Con_Matsu_H = (3.33E+14, 37363, 0)
A_Matsu_H, ER_Matsu_H, n_Matsu_H = Rate_Con_Matsu_H

# Matsugi TFE rate constants ---- With low pressure falloff at 1-4 bars
Rate_Con_Matsu_L = (5.71E+46, 47073, -9.341)
A_Matsu_L, ER_Matsu_L, n_Matsu_L = Rate_Con_Matsu_L 

# Lifshitz CPCN rate constants ---- New
Rate_Con_Lifs_N = (3.20E+14, 29106.228, 0)
A_Lifs_N, ER_Lifs_N, n_Lifs_N = Rate_Con_Lifs_N


# psi to atm conversion ---- 1 psi = ? atm
psi_atm=0.068046

####################################################################################

HPST=True

LPST=False

# pressure trace scaling. If nothing special, set it to 1
# shock pressure trace adjustment scaling factor for HPST experiments before Jai (Sep 2017)
# whether to use pressure scaling for the pressure trace. If not, let p_scaling be 1

p_scaling = 1.9138



# For calculations of the average temperature through chemical thermometer when n factor is not 0, 
# set a temperature range for simulations with constant pressure constant reaction time (in Kelvin).

Tl, Tu = 900, 2000

input_species=['AR', 'C10H16']
input_mole_frac=[1-0.000072, 0.000072]

# Species to analyze (it has to be chosen in CKSolnList.txt in src). The names should be exactly the same with ones used in 
# CHEMKIN model
specieslist=['C10H16','CH4','C2H4','C2H6','C2H2','C3H6','aC3H4','pC3H4','C4H81','C4H6','iC4H8','C5H6','C6H6','C6H5CH3',
            'C4H2','C4H4']

#########################################################################################

# Working folder
dir_work=folder_address+r"\Simulation_results"

# Pressure trace folder
dir_pressure=folder_address+r"\Pressure traces"



# Shock parameters summary file as input information about shocks

### File address
dir_shock_info=folder_address
fname_shock_info=r"HPST-JP10-04232018.xlsx"


### Target excel sheet in the file
sheet_shock_info=r"HPST-JP10-72ppm25atm"

### Header row location (shock_infor_header) on excel sheet starting from 1. input_
shock_info_header_excel_row=1




# A template folder that has the files you need run the chemkin program using a specific mechanism in it
# The files in this folder needs to be changed if you use another mechanism
src=folder_address+r"\template folder"

# Output file locations

# Computation summary output
dir_out_summary=folder_address+r"\HPST-JP10-25atm-changing-pressure-06212018.csv"
dir_p_out=dir_pressure+'\\processed'


###########################################################################################
# A little processing of the above inputs

file_shock_info=dir_shock_info+"\\"+fname_shock_info
input_header=shock_info_header_excel_row-1

###########################################################################################

df_info = pd.read_excel(file_shock_info, sheet_name=sheet_shock_info, header=input_header)
print(df_info.head(5))

###########################################################################################

# Make new column for initial temperature T5(0)
#df_info['T0']=0

speciesmatrix=np.empty((len(df_info['Shock Name']),len(specieslist)))
speciesmatrixA=np.empty((len(df_info['Shock Name']),len(specieslist)))

# Make new folder for output pressure trace data if that folder does not exist
if not os.path.exists(dir_p_out):
    os.makedirs(dir_p_out)

###########################################################################################
    

# Local linear kernel regression model. You can adjust the b array easily from the input or directly inside the function. 
# And the choice of b array (especially the earlier points) and x will directly affect the result of back calculation of T5(0).
# t is time. p is pressure. The default setting b=0.0001 is suited for t in seconds unit.

def local_linear_kernel_regression(t, p, x="auto", b=0.0001):
    
    if x=="auto":
        x=np.linspace(t[0], t[-1], 100)
        
    elif isinstance(x, (list, tuple, np.ndarray)):
        pass

    # You can adjust the equation here. Choose whatever curve of b along x, if you don't choose to do it in the input.
    # The function we use now is a shifted, flipped, and scaled logistic function. 
    if not isinstance(b, (list, tuple, np.ndarray)):
        b=5*b/(1+np.exp((x-t[0])/(0.8*b)))+b
    else:
        b=b
    
    # Kernels here
    try:
        k=np.array([np.exp(-(x-t[i])**2/(2*b**2)) for i in range(len(t))])
    except:
        print("error in calculating Gaussian kernels. Please check the input data to local_linear_kernel_regression()")
    
    # Below are calculating a formula you can find in Elements of Statistical Learning
    
    Onemat=np.array([1 for i in range(len(t))])

    B=np.transpose(np.stack([Onemat,t], axis=0))

    y=np.zeros(len(x))


    for i in range(len(x)):

        b=np.array([1,x[i]])
        W=diags(k[:,i],0)
        temp=np.transpose(B).dot(W*B)
        temp=np.linalg.inv(temp)
        #temp=np.linalg.inv(np.linalg.multi_dot([np.transpose(B),W,B]))
        y[i]=np.transpose(b).dot(temp).dot(np.transpose(B)).dot(W*p)
    
    return (x,y)


# Define a function for line cut to find the shock location and separate pre and post shock region
# So far this has been proved to be more reliable than K-means clustering.

def line_cut(x,y,y_cut):
    ind=0
    for i in range(len(x)-1):       
        if (y_cut-y[i])*(y_cut-y[i+1])<=0:
            interpfun=interp1d(y[i:(i+2)],x[i:(i+2)])
            x_cut=interpfun(y_cut)
            ind=i
            break
    return (x_cut, ind)   



# Now, we will use the kernel smoothing (regression) function and line cut function to do pressure trace smoothing of end wall P

def pressure_processing_1shock(t,p,p1,pres_scaling_factor=1, b='auto', kernel_size_medfilt=301, xsize=100,
                               show_figure_medfilt=1, show_figure_final=1, save_figure=None):
    if b=='auto':
        b1=0.00007

    elif not isinstance(b, (list, tuple, np.ndarray)):
        b1=float(b)

    else:
        pass

    
    if isinstance(p1, (list, tuple, np.ndarray)):
        print("Error. Check the size of p1.")
        return
    
    if len(t)!=len(p):
        print("Error. Sizes of t and p do not match!")
        
  
    # An approach of shock point detection using line cut for non-end wall locations
    #Locate again the shock points
    p_initial_smoothed=medfilt( p, kernel_size=101)
    ref_p_cut=0.4*p_initial_smoothed.max()
    ref_shockpoint=line_cut(t,p,ref_p_cut)[1]
    
    if abs(ref_p_cut)<p1:
        print("Error when detecting incident shock and reflected shock")
        plt.figure()
        plt.plot(t, p)
        plt.show()        
        raise
    
    else:
        pass
    
    # Processing raw data to fit to actual initial pressure baseline
    # Note that p1 also has uncertainty. And we don't know about it yet. The value is most likely larger than the value given here.
    # We may deal with this uncertainty outside this pressure processing function.
    p_baseline=np.average(p[:(ref_shockpoint-500)])
#    t_full=t  # raw data t
    p_full=p+p1-p_baseline  # raw pressure data with p1 as baseline


    # Regressed data initialization.
    # P1, 0th cluster
    x0=np.linspace(t[0],t[ref_shockpoint],100)  # 
    y0=np.zeros(100)+p1

    # P4  1st cluster
    x1=np.linspace(t[ref_shockpoint], t[-1], xsize) 
    y1=np.zeros(xsize)+p1


    #original code
    #t, p = t[inc_shockpoint:]-t[inc_shockpoint], p[inc_shockpoint:]+p1-p_baseline

    #""""
    #if __name__=="__call__":
    #    plt.plot(t, p, label='P5')
    #    plt.show()    
    #

    
    p0_smooth=medfilt( p_full[0:ref_shockpoint], kernel_size=kernel_size_medfilt)

    p1_smooth=medfilt( p_full[ref_shockpoint:], kernel_size=kernel_size_medfilt)

    p_smooth=np.concatenate((p0_smooth,p1_smooth),axis=0)

    if show_figure_medfilt==1:
        plt.figure()
        plt.plot(t, p_full,t,p_smooth)
        plt.xlim(t[ref_shockpoint-500],t[ref_shockpoint+500])
        plt.xlabel('Time (s)')
        plt.ylabel('Pressure (psi)')
        plt.show()
    else:
        pass

    y1=local_linear_kernel_regression(t[(ref_shockpoint+700):], p_full[(ref_shockpoint+700):], x=x1, b=b1)[1]

    x=np.concatenate((x0,x1),axis=0)
    y=np.concatenate((y0,y1),axis=0)
    
    if show_figure_final==1:
        plt.figure()
        plt.plot(t, p_full,x,y)
        plt.xlim(t[ref_shockpoint-500],t[ref_shockpoint+20000])
        plt.xlabel('Time (s)')
        plt.ylabel('Pressure (psi)')
        plt.legend()
        if save_figure is not None:
            plt.savefig(save_figure)
        else:
            pass
        plt.show()
    else:
        pass
        plt.gcf().clear()
    return(t[ref_shockpoint], t[ref_shockpoint], x, y, p_baseline)

################################################################################################################
# Given initial T5 P5, and pressure trace, get temperature trace (assuming specific heat ratio is fixed).
def GetTempTrace(tpdata,Tideal,gamma=1.667):
    g=Tideal*np.power(tpdata[:,1]/tpdata[0,1],1-(1/gamma))
    return np.transpose(np.array([tpdata[:,0],g]))    

# Get the integral that sums the "reaction extent" from temperature trace
def getintegral(ER,tp,T0,gamma=1.667,n=0.0):
    T=GetTempTrace(tp,T0,gamma)[:,1]
    Ex=np.exp(-ER/T)*np.power(T,n)
    #print(np.shape(Ex))
    #print(np.shape(tp[:,0]))
    J=np.trapz(Ex,x=tp[:,0])
    return J

# Back calculation based on average temperature and reaction time
def back_cal_T(ER,tp,Tavg,tao,gamma=1.667,n=0.0):
    Tavg=float(Tavg)
    Ta=float(Tavg-100)
    Tb=float(Tavg+100)
    T0=float((Ta+Tb)/2)
    J=float(0)
    Ja=getintegral(ER,tp,Ta,gamma,n)
    Jb=getintegral(ER,tp,Tb,gamma,n)
#    total reaction progress without A
    Javg=tao*np.exp(-ER/Tavg)*np.power(Tavg,n)
    while ((abs(J-Javg)/Javg)>0.0001):    
        print("J: "+str(J)+',   Javg: '+str(Javg))
        T0=(Ta+Tb)/2
        J=getintegral(ER,tp,T0,gamma,n)
        #print('ER: '+str(ER)+"  T0: "+str(T0)+"  J: "+str(J))        
        Ja=getintegral(ER,tp,Ta,gamma,n)
        Jb=getintegral(ER,tp,Tb,gamma,n)
        if (J-Javg)*(Ja-Javg)<0:
            Tb=T0
        elif (J-Javg)*(Jb-Javg)<0:
            Ta=T0
        else:
            return 0
            break 
    else:
        print("J: "+str(J)+',   Javg: '+str(Javg))
        return T0

    
    


def getTavg(A,ER,x,tao,Ta,Tb,n=0.0):
    funct=lambda T: abs(1-x-np.exp(-A*np.power(T,n)*np.exp(-ER/T)*tao))
    Tarray=np.linspace(Ta,Tb,1000)
    residual=list(map(funct,Tarray))
    Min_ind=np.argmin(residual)
    Tguess=Tarray[Min_ind]
    Tavg=fsolve(funct,Tguess)
    return Tavg[0]

########################################################################################



# Below is bulk run
for i in df_info.index:
    print(df_info['Shock Name'][i])
    # For Xu's shocks, this was how it's named in pressure trace file from HPST because we didn't extract side wall traces at that time
    P_file_name='Shock'+str(int(df_info['Shock No'][i]))
    
   
    path=dir_work+r"\\"+P_file_name
    
    # For Jai's shocks, and every relatively flate profile shocks, the side wall pressure traces can be extracted. Thus,
    # I would suggest using the format of shock??p?. p9 represents the end wall trace.
    #P_file_name='shock'+str(int(df_info['Shock No'][i])).zfill(2)+'p9'
    
    # Set address of pressure trace
    if HPST==True:
        P_address=dir_pressure+"\\"+P_file_name
    elif HPST==False:
        P_address=dir_pressure+"\\"+P_file_name+r'.txt'
        
    # Open pressure trace file
    try:
        open(P_address)
    except IOError:
        print('IOError: File does not exist or problem opening the file '+fileaddress)
    except:
        print('Problem opening the file '+fileaddress+', Not IOError')
    
    # Read pressure trace and process it
    data = np.genfromtxt(P_address)
    t=data[:,0]
    if HPST:
        p=data[:,1]
    elif LPST:
        p=data[:,8]
    
    p1 = df_info['P1 (psi)'][i]
    T5 = df_info['T5_Cal (K)'][i]
    T5_0 = df_info['T5_0 (K)'][i]
    # for later use
    reaction_time = df_info['Reaction Time (s)'][i]
    #t_end = df_info['Reaction Time (s)'][i]
    t_end=0.010
    ## Before pressure trace processing, set the location to save the figure.
    figure_address = dir_p_out+'\\'+P_file_name+".png"
    p=p*p_scaling
    _, _, t, p, _ = pressure_processing_1shock(t, p, p1, pres_scaling_factor=1, b=0.0001, kernel_size_medfilt=101, 
                                               xsize=200, show_figure_medfilt=0, show_figure_final=1, save_figure=figure_address)
   # except:
    #    print("Error processing raw pressure trace.")
    #    continue
        
   
    # Note that the calibrated temperature can be obtained from getTavg()
    # T5_0 is the initial temperature after reflected shock
    
    if os.path.exists(path)==False:
        print('copying template folder')
        shutil.copytree(src,
                    path)
        print('Completed copying template folder')
    else:
        src_files = os.listdir(src)
        for file_name in src_files:
            full_file_name = os.path.join(src, file_name)
            if (os.path.isfile(full_file_name)):
                shutil.copy(full_file_name, path)
    try:
        testshock = ds.shock(path, P_file_name, t[100:], p[100:]*psi_atm, T5_0, t_end, input_species, input_mole_frac)        
    except:
        print(ShocknameA+"has problem setting up.")
        shocksA.append(None)
#        VaryingPresYield_C2H4.append(None)
#        Tlist=Tlist.append(None)
        continue
    testshock.SetCPinputfile()
    csr.runshockafterpreproc(reactin=testshock.reactin_path,
                 envvars=testshock.directory+r"\\chemkinpro_env.txt",work_dir=testshock.directory,
                 getsolnlist=False)
    #xch4=testshock.QuickExtractppm(species='CH4')
    #xc2h4=testshock.QuickExtractppm(species='C2H4')
    #xc3h6=testshock.QuickExtractppm(species='C3H6')
    #xc4h81=testshock.QuickExtractppm(species='C4H81')
    #xic4h8=testshock.QuickExtractppm(species='iC4H8')
    for j in range(len(specieslist)):
        speciesmatrix[i,j]=testshock.QuickExtractppm(species=specieslist[j])
    

    tp=np.column_stack([t,p])
#    Tavg=df_info['T5_avg_pointwise (K)'][i]
#    tao=df_info['Reaction Time (s)'][i]
#    #try:
#    T0 = back_cal_T(rate_con[1], tp[100:,:], Tavg, tao, gamma=1.667, n=rate_con[2])
#    #except:
#     #   print("Error in back calculating")
#     #   continue
    
#    # Give values to the output variables
#    df_info['T0'][i]=T0
    np.savetxt(dir_p_out+'\\'+P_file_name+"_smoothed.csv", tp)
    
df_species=pd.DataFrame(data=speciesmatrix, index=df_info['Shock Name'][:], columns=specieslist)

df_species.to_csv(dir_out_summary)