import sys
import os
import numpy as np
import fileinput
import shutil
import pandas as pd
import xlrd
import PressureTraceProcessing as ptp

class shock(object):
    
    def __init__(self, directory,shockno,time,P,T,endt,species,fractions):
        self.directory=directory
        self.shockno=shockno
        self.time=time
        self.P=P
        self.T=T
        self.endt=endt
        self.species=species
        self.fractions=fractions
        self.reactin_path=self.directory+r'\\'+self.shockno+'.inp'
    
    Endppm_C2H4=0 
    Endppm_CH4=0 
    Endppm_C3H6=0 
        
#    def extractsolution(self):
#        CKsoln_path=self.directory+"\\CKSoln.ckcsv"
#        if os.path.exists(CKsoln_path):
#            pass
#        else:
#            print("Solution data CKSoln.ckcsv not found in the folder")
#            return
#        with open(CKsoln_path,'r') as solutions:
#            for line in solutions:
#                dlist=line.split(',')
#                if len(dlist)>10:
    
    def QuickExtractC2H4(self):
        CKsoln_path=self.directory+"\\CKSoln.ckcsv"
        if os.path.exists(CKsoln_path):
            pass
        else:
            print("Solution data CKSoln.ckcsv not found in the folder")
            return
        with open(CKsoln_path,'r') as solutions:
            for line in solutions:
                if line.find('Ppm_C2H4,')!=-1:
                    dlist=line.split(',')
                    break                    
        self.Endppm_C2H4=float(dlist[len(dlist)-1])
        return self.Endppm_C2H4

    def QuickExtractC3H6(self):
        CKsoln_path=self.directory+"\\CKSoln.ckcsv"
        if os.path.exists(CKsoln_path):
            pass
        else:
            print("Solution data CKSoln.ckcsv not found in the folder")
            return
        with open(CKsoln_path,'r') as solutions:
            for line in solutions:
                if line.find('Ppm_C3H6,')!=-1:
                    dlist=line.split(',')
                    break                    
        self.Endppm_C3H6=float(dlist[len(dlist)-1])
        return self.Endppm_C3H6
 
    def QuickExtractCH4(self):
        CKsoln_path=self.directory+"\\CKSoln.ckcsv"
        if os.path.exists(CKsoln_path):
            pass
        else:
            print("Solution data CKSoln.ckcsv not found in the folder")
            return
        with open(CKsoln_path,'r') as solutions:
            for line in solutions:
                if line.find('Ppm_CH4,')!=-1:
                    dlist=line.split(',')
                    break                    
        self.Endppm_CH4=float(dlist[len(dlist)-1])
        return self.Endppm_CH4        
    
    def QuickExtractppm(self,species='C2H4'):
        CKsoln_path=self.directory+"\\CKSoln.ckcsv"
        if os.path.exists(CKsoln_path):
            pass
        else:
            print("Solution data CKSoln.ckcsv not found in the folder")
            return
        with open(CKsoln_path,'r') as solutions:
            for line in solutions:
                if line.find('Ppm_'+species+',')!=-1:
                    dlist=line.split(',')
                    break                    
        Endppm=float(dlist[len(dlist)-1])
        return Endppm        


    def SetCPinputfile(self):
        print('Setting up '+self.shockno)
        if not os.path.exists(self.directory):
            os.makedirs(self.directory)
        else:
            print("Target directory exists")
        file_path=self.directory+r'\\'+self.shockno+'.inp'
        print (file_path)
    #    filein="C:\Python codes\Tests\HomogeneousReactor1.inp"
    #    shutil.copyfile(filetemp,filein)
    #    T=1500
    #    endt=0.004
    #    species=['AR', 'POSF10325']
    #    fractions=[0.993644, 0.006356]
        fractions=[(1/sum(self.fractions))*k for k in self.fractions]
        seq=['ENRG   ! Solve Gas Energy Equation\n',
             'TRAN   ! Transient Solver\n',
             'QLOS 0.0   ! Heat Loss (cal/sec)\n',
             'ADAP   ! Save Additional Adaptive Points\n',
             'GFAC 1.0   ! Gas Reaction Rate Multiplier\n']   #You can change seq based on what you want, but you should not write END in seq in any case
        filein=open(file_path,'w')
        filein.writelines(seq)
    #    !!The following may cost a large amount of time in writing the pressure trace profile if time and P are very large
        for i in np.arange(0,len(self.time)):
            filein.write('PPRO '+str(self.time[i])+' '+str(self.P[i])+' ! Pressure (atm)\n')
        filein.write('TEMP '+str(self.T)+' ! Temperature(K)\n')
        filein.write('TIME '+str(self.endt)+' ! End Time(sec)\n')
        for i in np.arange(0,len(fractions)):
            filein.write('REAC '+str(self.species[i])+' '+str(fractions[i])+' ! Reactant Fraction (mole fraction)\n')
        filein.write('END')
        return 1        

        
    
    def getendresults(soln_file="CKSoln.ckcsv"):
        """Check if file exists in directory"""
        if os.path.exists(soln_file)==False:
            sys.exit(soln_file+" not found in working directory, CHEMKIN may have\
                     failed to execute.")
        
        """initialize labels and data vectors"""
        Temp = "Temp Not Found"
        labels = []
        final_dat = []
        """open file"""    
        with open(soln_file,'r') as solutions:
            
          """read each line fot the file""" 
          for line in solutions:
              """for lines containing "ppm" """
              if line.find("Ppm")!=-1 or line.find("ppm")!=-1:
                  """split each line into a data list"""
                  data = line.split(sep=", ")
                  #print(data[0]+": "+data[len(data)-1])
                  """append labels and final_data with values"""
                  labels.append(data[0])
                  final_dat.append(float(data[len(data)-1]))
              elif line.find("Temperature")!=-1:
                  """extract the initial temperature"""
                  Temp = line.split(", ")[2]
              else:
                  continue    
        return(Temp,labels,final_dat)

"""allows script to be imported without execution"""
    





def QuickExtractppm(directory, species='C2H4'):
    CKsoln_path=directory+"\\CKSoln.ckcsv"
    if os.path.exists(CKsoln_path):
        pass
    else:
        print("Solution data CKSoln.ckcsv not found in the folder")
        return
    with open(CKsoln_path,'r') as solutions:
        for line in solutions:
            if line.find('Ppm_'+species+',')!=-1:
                dlist=line.split(',')
                break                    
    Endppm=float(dlist[len(dlist)-1])
    return Endppm            

    
#        
if __name__=='__main__':
#    p1=4
#    tpdata=ptp.PressureConvert(r"02062017-GevoATJ-2",r"C:\Python_Codes\Chemkintest",p1)
#    print(ptp.GetNominal(tpdata,p1))
#    TTT=ptp.GetTempTrace(tpdata,1300,1.667)
#    folder=r"C:\\Python_Codes\\Chemkintest"
#    Rname=r"homotest1"
 #   testshock=shock(folder,Rname,tpdata[0,:],tpdata[1,:],1200,0.004,['AR', 'POSF10325'],
#                   [0.993644, 0.006356])
    
 #   testshock.SetCPinputfile()
     h=0
    
        