# -*- coding: utf-8 -*-
"""
Created on Thu Jul 13 12:51:56 2017

@author: Xu
"""

import os
import numpy as np
import fileinput
import shutil
import csv
import matplotlib.pyplot as plt
import time
from sklearn.cluster import KMeans
from scipy.interpolate import interp1d
import chemkinsinglerun as csr
from scipy.optimize import fsolve


# 
def PressureConvert(shockno=r"02062017-GevoATJ-2",filelocation=r"C:\Python_Codes\Chemkintest",
                    p1=0,b=0.0001,xsize=100,filetype='txt',index=8, presscalingfactor=1, fullshockfig=False, fullshockreturn=False, legendcontent=False, savefig=False):
#input unit is psi (including p1), output unit is atm
    if filetype==None:
        fileaddress=filelocation+'\\'+shockno
    else:
        fileaddress=filelocation+'\\'+shockno+'.'+filetype
    try:
        open(fileaddress)
    except IOError:
        print('IOError: File does not exist or problem opening the file '+fileaddress)
        return 0
    except:
        print('Problem opening the file '+fileaddress+', Not IOError')
        return 0
    
    pictureaddress=filelocation+'\\'+shockno+'.jpg'
    
    data=np.genfromtxt(fileaddress)
    p=data[:,index]*presscalingfactor  #convert to atm
    t=data[:,0]
    p1=p1
    del data
    count=int(len(t)*0.3)
    xl=t[0:count]
    yl=p[0:count]   
    mat=np.transpose(np.stack((xl,yl)))
    kmeans = KMeans(n_clusters=2, random_state=0).fit(mat)
    label0=np.where(kmeans.labels_==0)
    if label0[0][-1]!=len(xl)-1:
        shockpoint=label0[0][-1]+1
    elif len(label0[0])!=len(xl):
        shockpoint=np.where(kmeans.labels_==1)[0][0]
    else:
        print("Problem with clustering. Check your pressure profile and give the shock time yourself")
        return (0)   #This part will be changed later on
    
    t0=np.linspace(0,t[shockpoint],100)
    p_baseline=np.average(p[:(shockpoint-500)])
    p0=np.zeros(100)+p1
    tfull_raw=t
    pfull_raw=p+p1-p_baseline
    
    t, p = t[shockpoint:]-t[shockpoint], p[shockpoint:]+p1-p_baseline
    
    if __name__=="__call__":
        plt.plot(t, p, label='P5')
        plt.show()    
    
    x=np.linspace(0,max(t),xsize)
    k=np.ndarray(shape=(len(t),len(x)),dtype=float)
    start_time = time.time()
    k=np.array([np.exp(-(x-t[i])**2/(2*b**2)) for i in range(len(t))])
    print("--- %s seconds ---" % (time.time() - start_time))
    pnew=np.array([np.dot(k[:,j],p)/np.sum(k[:,j]) for j in range(len(x))])
    print("--- %s seconds ---" % (time.time() - start_time))
    
    ts=np.zeros(100)+t[shockpoint]
    ps=np.linspace(p1,pnew[0],100)
    
    tfull=np.concatenate((t0,ts,x+t[shockpoint]),axis=0)
    pfull=np.concatenate((p0,ps,pnew),axis=0)
    
    if fullshockfig==False:
        plt.plot(x,pnew, t,p, label='P - '+shockno)
    else:
        #plot shock profile including preshock region
        plt.plot(tfull_raw, pfull_raw, tfull,pfull, label='P - '+shockno)
    plt.legend()
#    plt.title('Pressure trace of '+shockno)
    plt.xlabel('t(s)')
    plt.ylabel('P(psi)')
#    plt.show()
    if savefig==True:
        plt.savefig(pictureaddress)
    elif savefig==False:
        pass
    plt.gcf().clear()
    
    if fullshockreturn==False:
        return np.array([x,pnew])
    else:
        #return the shock wave profile including preshock region
        return np.array([tfull,pfull])


def GetNominal(tpdata,p1,c=0.8):
    pmax=max(tpdata[1,:])
    pnominal=c*(pmax-p1)+p1
    for i in reversed(range(len(tpdata[1,:])-1)):       
        if (pnominal-tpdata[1,i])*(pnominal-tpdata[1,i+1])<0:
            interpfun=interp1d(tpdata[1,i:(i+2)],tpdata[0,i:(i+2)])
            tnominal=interpfun(pnominal)
            break
        
    return np.array([tnominal,pnominal,pmax/tpdata[1,0]-1])

def GetNominal2(tpdata,p1):
    pmax=max(tpdata[1,:])
    pnominal=0.8*(pmax-p1)+p1
    pnominal_highT=0.6*(pmax-p1)+p1
    tnominal=0
    for i in reversed(range(len(tpdata[1,:])-1)):       
        if (pnominal_highT-tpdata[1,i])*(pnominal_highT-tpdata[1,i+1])<0:
            interpfun=interp1d(tpdata[1,i:(i+2)],tpdata[0,i:(i+2)])
            tnominal=interpfun(pnominal_highT)
            break
        
    return np.array([tnominal,pnominal,pmax/tpdata[1,0]-1])


def GetTempTrace(tpdata,Tideal,gamma):
    g=Tideal*np.power(tpdata[1,:]/tpdata[1,0],1-(1/gamma))
    return np.array([tpdata[0,:],g])    

def getintegral(ER,tp,T0,gamma=1.66,n=0.0):
    T=GetTempTrace(tp,T0,gamma)[1,:]
    Ex=np.exp(-ER/T)*np.power(T,n)
    J=np.trapz(Ex,x=tp[0,:])
    return J

#Back calculation based on average temperature and reaction time
def backcalT_1(ER,tp,Tavg,tao,gamma=1.667,n=0.0):
    Ta=Tavg-100
    Tb=Tavg+100
    T0=(Ta+Tb)/2
    J=0
    Ja=getintegral(ER,tp,Ta,gamma,n)
    Jb=getintegral(ER,tp,Tb,gamma,n)
#    total reaction progress without A
    Javg=tao*np.exp(-ER/Tavg)*np.power(Tavg,n)
    while ((abs(J-Javg)/Javg)>0.001):    
        T0=(Ta+Tb)/2
        J=getintegral(ER,tp,T0,gamma,n)
        Ja=getintegral(ER,tp,Ta,gamma,n)
        Jb=getintegral(ER,tp,Tb,gamma,n)
        if (J-Javg)*(Ja-Javg)<0:
            Tb=T0
        elif (J-Javg)*(Jb-Javg)<0:
            Ta=T0
        else:
            return 0
            break 
    else:
        return T0


def getTavg(A,ER,x,tao,Ta,Tb,n=0.0):
    funct=lambda T: abs(1-x-np.exp(-A*np.power(T,n)*np.exp(-ER/T)*tao))
    Tarray=np.linspace(Ta,Tb,1000)
    residual=map(funct,Tarray)
    Min_ind=np.argmin(residual)
    Tguess=Tarray[Min_ind]
    Tavg=fsolve(funct,Tguess)
    return Tavg[0]



                            
        
        

    
    
    
if __name__=="__main__":
#    p1=9.71
#    tpdata=PressureConvert(r"04102017-JetA2-4",r"C:\Python_Codes\CHEMKINsimulations\JetA2_2017\New folder\Pressure traces\LPST\JetA2 2017",p1,xsize=1000,fullshockfig=True, fullshockreturn=True)
#    np.savetxt(r'C:\Python_Codes\CHEMKINsimulations\JetA2_2017\New folder\Pressure traces\LPST\JetA2 2017\Processed_04102017-JetA2-4.txt', np.transpose(tpdata))
#    print(GetNominal(tpdata,p1))
#    TTT=GetTempTrace(tpdata,1300,1.66)

#    A=3.33*(10**14)
#    ER=37363
#    x=0.482657603
#    tao=0.0024067
#    n=0.0
#    T=getTavg(A,ER,x,tao,1100,1600,n=n)
#    print(T)
#        Shockname=
#        ptp.PressureConvert(Shockname,PTraceFolder,p1,b=0.0001, xsize=1000, index=8,fullshockfig=True, fullshockreturn=True,filetype='txt', savefig=True)

    pass












